mod grpc;
mod http;
mod state;

use std::net::SocketAddr;

use chrono::{Datelike, Timelike};
use clap::{Parser, Subcommand};
use grpc::{GrpcServer, PublishEventRequest};
use http::HttpServer;
use mad::Mad;
use state::SharedState;

#[derive(Parser)]
#[command(author, version, about, long_about = None, subcommand_required = true)]
struct Command {
    #[command(subcommand)]
    command: Option<Commands>,
}

#[derive(Subcommand)]
enum Commands {
    Serve {
        #[arg(env = "SERVICE_HOST", long, default_value = "127.0.0.1:3000")]
        host: SocketAddr,
        #[arg(env = "GRPC_SERVICE_HOST", long, default_value = "127.0.0.1:7900")]
        grpc_host: SocketAddr,
    },

    Client {
        #[arg(env = "SERVICE_HOST", long, default_value = "http://127.0.0.1:3000")]
        host: String,
        #[arg(
            env = "GRPC_SERVICE_HOST",
            long,
            default_value = "http://127.0.0.1:7900"
        )]
        grpc_host: String,

        #[command(subcommand)]
        commands: ClientCommands,
    },
}

#[derive(Subcommand)]
enum ClientCommands {
    PublishEvent {
        #[arg(long)]
        topic: String,
        #[arg(long)]
        key: String,
        #[arg(long)]
        value: String,
    },
}

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    dotenv::dotenv().ok();
    tracing_subscriber::fmt::init();

    let cli = Command::parse();

    match cli.command.unwrap() {
        Commands::Serve { host, grpc_host } => {
            tracing::info!("Starting service");
            let state = SharedState::new().await?;
            Mad::builder()
                .add(HttpServer::new(&state, host))
                .add(GrpcServer::new(&state, grpc_host))
                .run()
                .await?;
        }
        Commands::Client {
            commands,
            host,
            grpc_host,
        } => match commands {
            ClientCommands::PublishEvent { topic, key, value } => {
                let mut client =
                    crate::grpc::no_data_client::NoDataClient::connect(grpc_host).await?;

                let timestamp = chrono::Utc::now();

                let res = client
                    .publish_event(PublishEventRequest {
                        topic,
                        published: Some(prost_types::Timestamp::date_time_nanos(
                            timestamp.year() as i64,
                            timestamp.month() as u8,
                            timestamp.day() as u8,
                            timestamp.hour() as u8,
                            timestamp.minute() as u8,
                            timestamp.second() as u8,
                            timestamp.nanosecond(),
                        )?),
                        key,
                        value: value.into_bytes(),
                    })
                    .await?;
            }
        },
    }

    Ok(())
}
